// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_authentication_bloc/src/domain/entities/account.dart';
import 'package:wyatt_authentication_bloc/src/domain/entities/account_wrapper.dart';
import 'package:wyatt_form_bloc/wyatt_form_bloc.dart';

abstract class AuthenticationRepository<T> extends BaseRepository {
  FormRepository get formRepository;

  /// {@template signup}
  /// Creates a new user with the provided [email] and [password].
  ///
  /// Returns the newly created user's unique identifier.
  ///
  /// Throws a SignUpWithEmailAndPasswordFailureInterface if
  /// an exception occurs.
  /// {@endtemplate}
  FutureResult<Account> signUp({
    required String email,
    required String password,
  });

  /// {@template send_email_verification}
  /// Sends verification email to the account email.
  ///
  /// Throws a SendEmailVerificationFailureInterface if an exception occurs.
  /// {@endtemplate}
  FutureResult<void> sendEmailVerification();

  /// {@template send_password_reset_email}
  /// Sends a password reset email to the provided [email].
  ///
  /// Throws a SendPasswordResetEmailFailureInterface if an exception occurs.
  /// {@endtemplate}
  FutureResult<void> sendPasswordResetEmail({required String email});

  /// {@template confirm_password_reset}
  /// Confirms the password reset with the provided [newPassword] and [code].
  ///
  /// Throws a ConfirmPasswordResetFailureInterface if an exception occurs.
  /// {@endtemplate}
  FutureResult<void> confirmPasswordReset({
    required String code,
    required String newPassword,
  });

  /// {@template verify_password_reset_code}
  /// Verify password reset code.
  ///
  /// Throws a VerifyPasswordResetCodeFailureInterface if an exception occurs.
  /// {@endtemplate}
  FutureResult<bool> verifyPasswordResetCode({required String code});

  /// {@template signin_anom}
  /// Sign in anonymously.
  ///
  /// Throws a SignInAnonymouslyFailureInterface if an exception occurs.
  /// {@endtemplate}
  FutureResult<Account> signInAnonymously();

  /// {@template signin_pwd}
  /// Signs in with the provided [email] and [password].
  ///
  /// Throws a SignInWithEmailAndPasswordFailureInterface if
  /// an exception occurs.
  /// {@endtemplate}
  FutureResult<Account> signInWithEmailAndPassword({
    required String email,
    required String password,
  });

  /// {@template signout}
  /// Signs out the current user.
  /// It also clears the cache and the associated data.
  /// {@endtemplate}
  FutureResult<void> signOut();

  FutureResult<void> refresh();

  /// {@template stream_account}
  /// Stream of [AccountWrapper] which will emit the current account when
  /// the authentication state changes.
  ///
  /// Emits [AccountWrapper] with null [Account] if the user is not 
  /// authenticated.
  /// {@endtemplate}
  Stream<FutureResult<AccountWrapper<T>>> streamAccount();

  FutureResult<String> getIdentityToken();

  FutureResult<Account> getAccount();
  FutureResult<void> setAccount(Account account);

  FutureResult<T> getData();
  FutureResult<void> setData(T? data);

  FutureResult<AccountWrapper<T>> getCache();
  FutureResult<void> destroyCache();
}
