// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:async';
import 'dart:io';

import 'package:wyatt_cli_toolbox/src/sequences/sequences.dart';
import 'package:wyatt_cli_toolbox/src/spinners/spinners.dart';
import 'package:wyatt_cli_toolbox/src/stylish/stylish.dart';
import 'package:wyatt_cli_toolbox/src/workflow/skip_exception.dart';

enum ExitStatus { nil, success, fail, skip }

class Task<T> {
  Task({
    required this.name,
    String? id,
    SpinnerType? spinner,
    this.timer = true,
    this.task,
    this.onFinish,
    this.onError,
    this.onSkip,
  }) {
    if (id != null) {
      identifier = id;
    } else {
      identifier = name;
    }
    _isolateTask = IsolateTask(
      name: name,
      spinner: spinner ?? SpinnerType.dots,
      timer: timer,
    );
  }

  String name;
  late String identifier;
  bool timer;

  /// **Synchronous** function. Use experimental `waitFor()`
  /// if you have async calls.**
  T Function()? task;
  T? result;

  String Function(Object? o)? onFinish;
  String Function(Object e)? onError;
  String Function(SkipException e)? onSkip;

  late IsolateTask _isolateTask;
  static String lastPrint = '';

  IsolateTask get animation => _isolateTask;

  T? invoke() {
    final T Function()? t = task;
    if (t != null) {
      return t();
    }
  }

  static void print(String message) {
    //waitFor(Future<void>.delayed(Duration(milliseconds: 1)));
    lastPrint = message;
    stdout.write('\n');
    Sequences.moveDownAtStart(1);
    Sequences.clearEntireLine();
    stdout.write('   > $message'.dim());
    Sequences.moveUpAtStart(1);
  }

  static void clearPrint() {
    Sequences.moveDownAtStart(1);
    Sequences.clearEntireLine();
    Sequences.moveUpAtStart(1);
  }
}

class IsolateTask {
  IsolateTask({
    required this.name,
    required SpinnerType spinner,
    required this.timer,
  }) {
    _indicator = spinner.indicator;
  }

  final Stopwatch _stopwatch = Stopwatch();
  Timer? _timer;
  int _frame = 0;
  late Indicator _indicator;

  String name;
  bool timer;

  void run() {
    if (timer) {
      _stopwatch
        ..reset()
        ..start();
      _timer?.cancel();
    }

    _timer = Timer.periodic(
      Duration(milliseconds: _indicator.interval),
      (Timer t) {
        Sequences.clearEntireLine();
        _frame = (_frame + 1) % _indicator.frames.length;
        stdout.write('${_indicator.frames[_frame].green()} $name...');
        Sequences.moveHorizontal(1);
      },
    );
  }

  void stop(ExitStatus status, {String? reason}) {
    _stopwatch.stop();
    final String time =
        (_stopwatch.elapsed.inMilliseconds / 1000.0).toStringAsFixed(1);
    Sequences.clearEntireLine();
    String prefix = '';
    String message = '';
    if (status == ExitStatus.success) {
      prefix = '✓'.green();
      message = reason ?? 'Finished';
    } else if (status == ExitStatus.fail) {
      prefix = 'x'.red();
      message = reason ?? 'Failed';
    } else if (status == ExitStatus.skip) {
      prefix = '◊'.yellow();
      message = reason ?? 'Skipped';
    }
    stdout.write('$prefix $message ${timer ? '(${time}s)' : ''}\n');
    _timer?.cancel();
  }
}
